/* valaelementaccess.vala
 *
 * Copyright (C) 2006  Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Raffaele Sandrini <rasa@gmx.ch>
 */

#include "valaelementaccess.h"
#include "valasourcereference.h"
#include "valacodevisitor.h"

struct _ValaElementAccessPrivate {
	ValaExpression* _container;
	ValaExpression* _index;
};
#define VALA_ELEMENT_ACCESS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_ELEMENT_ACCESS, ValaElementAccessPrivate))
enum  {
	VALA_ELEMENT_ACCESS_DUMMY_PROPERTY,
	VALA_ELEMENT_ACCESS_CONTAINER,
	VALA_ELEMENT_ACCESS_INDEX,
};
static void vala_element_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_element_access_dispose (GObject * obj);

ValaElementAccess* vala_element_access_new (ValaExpression* _container, ValaExpression* _index, ValaSourceReference* source)
{
	g_return_val_if_fail (_container == NULL || VALA_IS_EXPRESSION (_container), NULL);
	g_return_val_if_fail (_index == NULL || VALA_IS_EXPRESSION (_index), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	GParameter * __params = g_new0 (GParameter, 3);
	GParameter * __params_it = __params;
	(__params_it->name = "container", g_value_init (&__params_it->value, VALA_TYPE_EXPRESSION), g_value_set_object (&__params_it->value, _container), __params_it++);
	(__params_it->name = "index", g_value_init (&__params_it->value, VALA_TYPE_EXPRESSION), g_value_set_object (&__params_it->value, _index), __params_it++);
	(__params_it->name = "source-reference", g_value_init (&__params_it->value, VALA_TYPE_SOURCE_REFERENCE), g_value_set_object (&__params_it->value, source), __params_it++);
	ValaElementAccess * self = g_object_newv (VALA_TYPE_ELEMENT_ACCESS, __params_it - __params, __params);
	return self;
}


static void vala_element_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaElementAccess * self = VALA_ELEMENT_ACCESS (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_element_access_get_container (self)), visitor);
	vala_code_node_accept (VALA_CODE_NODE (vala_element_access_get_index (self)), visitor);
	vala_code_visitor_visit_element_access (visitor, self);
}


ValaExpression* vala_element_access_get_container (ValaElementAccess* self)
{
	g_return_val_if_fail (VALA_IS_ELEMENT_ACCESS (self), NULL);
	return self->priv->_container;
}


void vala_element_access_set_container (ValaElementAccess* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (self));
	ValaExpression* __temp1 = NULL;
	self->priv->_container = (__temp1 = g_object_ref (value), (self->priv->_container == NULL ? NULL : (self->priv->_container = (g_object_unref (self->priv->_container), NULL))), __temp1);
}


ValaExpression* vala_element_access_get_index (ValaElementAccess* self)
{
	g_return_val_if_fail (VALA_IS_ELEMENT_ACCESS (self), NULL);
	return self->priv->_index;
}


void vala_element_access_set_index (ValaElementAccess* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (self));
	ValaExpression* __temp3 = NULL;
	self->priv->_index = (__temp3 = g_object_ref (value), (self->priv->_index == NULL ? NULL : (self->priv->_index = (g_object_unref (self->priv->_index), NULL))), __temp3);
}


void vala_element_access_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaElementAccess * self = VALA_ELEMENT_ACCESS (object);
	switch (property_id) {
		case VALA_ELEMENT_ACCESS_CONTAINER:
		g_value_set_object (value, vala_element_access_get_container (self));
		break;
		case VALA_ELEMENT_ACCESS_INDEX:
		g_value_set_object (value, vala_element_access_get_index (self));
		break;
	}
}


void vala_element_access_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaElementAccess * self = VALA_ELEMENT_ACCESS (object);
	switch (property_id) {
		case VALA_ELEMENT_ACCESS_CONTAINER:
		vala_element_access_set_container (self, g_value_get_object (value));
		break;
		case VALA_ELEMENT_ACCESS_INDEX:
		vala_element_access_set_index (self, g_value_get_object (value));
		break;
	}
}


static void vala_element_access_class_init (ValaElementAccessClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaElementAccessPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_element_access_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_element_access_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_element_access_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_element_access_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ELEMENT_ACCESS_CONTAINER, g_param_spec_object ("container", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ELEMENT_ACCESS_INDEX, g_param_spec_object ("index", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_element_access_init (ValaElementAccess * self)
{
	self->priv = VALA_ELEMENT_ACCESS_GET_PRIVATE (self);
}


static void vala_element_access_dispose (GObject * obj)
{
	ValaElementAccess * self = VALA_ELEMENT_ACCESS (obj);
	(self->priv->_container == NULL ? NULL : (self->priv->_container = (g_object_unref (self->priv->_container), NULL)));
	(self->priv->_index == NULL ? NULL : (self->priv->_index = (g_object_unref (self->priv->_index), NULL)));
	ValaElementAccessClass * klass;
	GObjectClass * parent_class;
	klass = VALA_ELEMENT_ACCESS_CLASS (g_type_class_peek (VALA_TYPE_ELEMENT_ACCESS));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_element_access_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaElementAccessClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_element_access_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaElementAccess), 0, (GInstanceInitFunc) vala_element_access_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaElementAccess", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



